@@ -6,14 +6,14 @@
  **
  ************************************************************************
  **
- **   Copyright (c): 2004-2010 by Axel Pauli (axel@kflog.org)
+ **   Copyright (c): 2004-2011 by Axel Pauli (axel@kflog.org)
  **
  **   This program is free software; you can redistribute it and/or modify
  **   it under the terms of the GNU General Public License as published by
  **   the Free Software Foundation; either version 2 of the License, or
  **   (at your option) any later version.
  **
- **   $Id: gpscon.cpp 4376 2010-11-03 19:07:56Z axel $
+ **   $Id: gpscon.cpp 4613 2011-04-30 11:48:18Z axel $
  **
  ***********************************************************************/
 
@@ -263,8 +263,8 @@ bool GpsCon::startGpsReceiving()
 #endif
         }
 
-      // we want to get a notification, if data are available on client side.
-      writeClientMessage(0, MSG_NTY );
+      // We switch on the data forwarding on the client side.
+      writeClientMessage(0, MSG_FGPS_ON );
       readClientMessage(0, msg);
 
       // remember last start time
@@ -356,8 +356,8 @@ void GpsCon::slot_StartGpsBtReceiving( bool ok,
 #endif
     }
 
-  // we want to get a notification, if data are available on client side.
-  writeClientMessage(0, MSG_NTY );
+  // We switch on the data forwarding on the client side.
+  writeClientMessage(0, MSG_FGPS_ON );
   readClientMessage(0, msg);
 
   // remember last start time
@@ -683,14 +683,6 @@ void GpsCon::slot_Timeout()
       // client not alive or start not desired
       return;
     }
-
-  if( GeneralConfig::instance()->getGpsDevice() != NMEASIM_DEVICE &&
-      lastQuery.elapsed () > ALIVE_TO )
-    {
-      // more as 15s no new data notification came in. We send a query to the
-      // client to be sure, that the notification has not gone lost.
-      queryClient();
-    }
 }
 
 /**
@@ -766,102 +758,95 @@ void GpsCon::slot_ListenEvent( int socket )
 }
 
 /**
- * This slot is triggered by the QT main loop and is used to handle the
- * notification events from the client.
+ * This slot is triggered by the QT main loop and is used to get the
+ * GPS or status data from the client.
  */
 void GpsCon::slot_NotificationEvent( int socket )
 {
-  static QString method = QString("GPSCon::slot_NotificationEvent(%1):").arg(socket);
+  QString method = QString("GPSCon::slot_NotificationEvent(%1):").arg(socket);
 
   // Disable client notifier if socket shall be read. Advised by Qt.
   clientNotifier->setEnabled( false );
 
-  QString msg = "";
-
-  // reads the notification message from the socket.
-  readClientMessage( 1, msg );
-
 #ifdef DEBUG
   qDebug("%s %s, got notification", method.toLatin1().data(), msg.toLatin1().data());
 #endif
 
-  if( msg != MSG_DA )
-    {
-      qWarning("%s unexpected notification message code %s",
-               method.toLatin1().data(), msg.toLatin1().data() );
-    }
-  else
-    {
-      queryClient();
-    }
+  getDataFromClient();
 
   // Enable client notifier after read.
   clientNotifier->setEnabled( true );
 }
 
 /**
- * Query the client, if NMEA records are available. If true, the data will
- * be hand over to the Cumulus process.
+ * Gets the GPS or status data from the client.
  */
-void GpsCon::queryClient()
+void GpsCon::getDataFromClient()
 {
-  if (server.getClientSock(0) == -1)
+  if( server.getClientSock( 1 ) == -1 )
     {
       // No connection to the client established.
       return;
     }
 
-  QString msg;
-  int loops = 250; // limit loops to avoid a dead lock
+  // QTime t; t.start();
 
-  // Now get all messages from the GPS client
-  while( loops-- )
+  int loops = 0;
+
+  while( loops++ < 250 )
     {
-      writeClientMessage(0, MSG_GM );
-      readClientMessage(0, msg);
+      // Check, if bytes are available in the receiver buffer because we
+      // use blocking IO.
+      int bytes = 0;
 
-      if (server.getClientSock(0) == -1)
+      // Number of bytes currently in the socket receiver buffer.
+      if( ioctl( server.getClientSock( 1 ), FIONREAD, &bytes) == -1 )
         {
-          // socket will be closed in case of any problems, e.g. client
-          // crash. we check that to avoid a dead lock here
+          qWarning() << "GpsCon::getDataFromClient():"
+                     << "ioctl() returns with ERROR: errno="
+                     << errno
+                     << "," << strerror(errno);
+          break;
+        }
+
+      if( bytes <= 0 )
+        {
+          break;
+        }
+
+      QString msg;
+
+      readClientMessage( 1, msg );
+
+      if( server.getClientSock( 1 ) == -1 )
+        {
+          // socket will be closed in case of any problems, e.g. client has
+          // crashed. we check that to avoid a dead lock here.
           return;
         }
 
-      if (msg.indexOf(MSG_RM) == 0)
+      if( msg.startsWith( MSG_GPS_DATA ) )
         {
-          // reply message received, 3 kinds are possible. remove message key
-          // and space separator before further processing
-          msg = msg.right(msg.length() - strlen(MSG_RM) - 1);
-
-          if (msg == MSG_CON_OFF) // GPS connection has gone off
-            {
-              emit gpsConnectionOff();
-              qDebug(MSG_CON_OFF);
-            }
-          else if ( msg == MSG_CON_ON ) // GPS connection has gone on
-            {
-              emit gpsConnectionOn();
-              qDebug(MSG_CON_ON);
-            }
-          else // GPS NMEA record
-            {
-              emit newSentence(msg);
-            }
-
-          continue;
+          msg = msg.right(msg.length() - strlen(MSG_GPS_DATA) - 1);
+          emit newSentence(msg);
         }
-
-      // no more messages are available on client
-      if (msg.indexOf(MSG_NEG) == 0)
+      else if (msg == MSG_CON_OFF) // GPS connection has gone off
         {
-          // no more data available
-          break;
+          emit gpsConnectionOff();
+          qDebug(MSG_CON_OFF);
+        }
+      else if ( msg == MSG_CON_ON ) // GPS connection has gone on
+        {
+          emit gpsConnectionOn();
+          qDebug(MSG_CON_ON);
+        }
+      else
+        {
+          qWarning() << "GpsCon::getDataFromClient(): Protocol Error!" << msg;
         }
     }
 
-  // renew notification subscription
-  writeClientMessage(0, MSG_NTY );
-  readClientMessage(0, msg);
+  // qDebug() << "MSG_GPS_DATA Loops" << loops << t.elapsed();
 
   // remember last start time
   lastQuery.start();
