@@ -7,7 +7,7 @@
 
     email                : axel@kflog.org
 
-    $Id: gpsnmea.cpp 4354 2010-10-26 21:11:10Z axel $
+    $Id: gpsnmea.cpp 4600 2011-04-29 20:49:33Z axel $
 
  ***************************************************************************
  *                                                                         *
@@ -70,13 +70,13 @@ QHash<QString, short> GpsNmea::gpsHash;
 
 GpsNmea::GpsNmea(QObject* parent) : QObject(parent)
 {
-  if ( ++instances > 1 )
+  if( instances > 0 )
     {
       // There exists already a class instance as singleton.
       return;
     }
 
-  ++instances;
+  instances++;
 
   resetDataObjects();
 
@@ -120,10 +120,7 @@ GpsNmea::GpsNmea(QObject* parent) : QObject(parent)
 GpsNmea::~GpsNmea()
 {
   // decrement instance counter
-  if ( --instances )
-    {
-      return;
-    }
+  instances--;
 
   // stop GPS client process
   if ( serial )
@@ -306,13 +303,13 @@ void GpsNmea::slot_sentence(const QString& sentenceIn)
     {
       // PFLAA receiving starts
       pflaaIsReceiving = true;
-      //qDebug() << "PFLAA receiving started";
+      // qDebug() << "PFLAA receiving started";
     }
   else if( pflaaIsReceiving == true )
     {
       // PFLAA receiving is finished
       pflaaIsReceiving = false;
-      //qDebug() << "PFLAA receiving finished";
+      // qDebug() << "PFLAA receiving finished";
       Flarm::instance()->collectPflaaFinished();
     }
 
